'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _window = require('./apis/window');

var _window2 = _interopRequireDefault(_window);

var _platform = require('./utils/platform');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var trapSet = false;
var currentUrl = null;
var defaultResolveMeta = function defaultResolveMeta(_ref) {
  var meta = _ref.meta;
  return meta;
};

exports.default = function () {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref2$resolveMeta = _ref2.resolveMeta,
      resolveMeta = _ref2$resolveMeta === undefined ? defaultResolveMeta : _ref2$resolveMeta,
      _ref2$api = _ref2.api,
      api = _ref2$api === undefined ? _window2.default : _ref2$api;

  return function (store) {
    var patched = !(0, _platform.needsPatching)(api.getUserAgent());
    // Since we want to control the browser beyond simple linear navigations
    // we need to trap it so that we can control the back button.
    // this is done by introducing a new page, so, when the user clicks the back
    // button, this is an event we can capture
    if (!trapSet) {
      var state = api.getState();
      // we start by testing that the current state stored in the browser is
      // not a previous session
      // If it isn't we set the trap
      if (!state || !Array.isArray(state)) {
        api.pushState({ ios: 'fix' }, null, api.getCurrentUrl());
      }
      trapSet = true;
    }

    // This means that when ever the user clicks back, we can not listen
    // for that event
    api.listenForPop(function () {
      if (patched) {
        var _ret = function () {
          // Fix for iOS 9.2 and below bug,
          // where initial view causes a popState

          // And if it happens, we can dispatch a travel event, which reverts to an
          // earlier commit, and then pushes that as a new page, to preserve the trap
          var url = document.location.pathname + document.location.search;
          if (url === currentUrl) {
            return {
              v: void 0
            };
          }

          Promise.resolve(store.dispatch({
            type: '@@history/TRAVEL'
          })).then(function () {
            currentUrl = url;
          });
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      } else {
        patched = true;
      }
    });

    // We want to liste for all actions on the store
    return function (next) {
      return function (action) {
        // And if we see a navigate request, which has not et been resolved
        if (action.type === '@@history/NAVIGATE' && !action.resolved) {
          // We dispatch a `before navigate` request to create a new empty commit
          store.dispatch(_extends({}, action, {
            type: '@@history/BEFORE_NAVIGATE',
            preloaded: (action.meta || {}).preloaded
          }));

          // And then resolves the meta data for the requested page
          return Promise.resolve(resolveMeta({
            url: action.url,
            meta: action.meta || {},
            store: store,
            context: {
              onSetTitle: function onSetTitle() {},
              onSetMeta: function onSetMeta() {}
            }
          })).then(function (meta) {
            // Once the meta data has been resolved we re-dispatch the navigation
            // action as resolved and with its meta data
            if (meta.redirect) {
              action.url = meta.redirect;
              return resolveMeta({
                url: meta.redirect,
                meta: action.meta || {},
                store: store,
                context: {
                  onSetTitle: function onSetTitle() {},
                  onSetMeta: function onSetMeta() {}
                }
              });
            } else {
              return meta;
            }
          }).then(function (meta) {
            var updatedAction = _extends({}, action, {
              meta: meta,
              preloaded: (action.meta || {}).preloaded,
              resolved: true
            });
            store.dispatch(updatedAction);
            currentUrl = document.location.pathname + document.location.search;
          });
        } else {
          return next(action);
        }
      };
    };
  };
};